<?php

declare(strict_types=1);

namespace Drupal\media_qualtrics\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\media_remote\Plugin\Field\FieldFormatter\MediaRemoteFormatterBase;

/**
 * Plugin implementation of the 'media_qualtrics' formatter.
 *
 * @FieldFormatter(
 *   id = "media_qualtrics",
 *   label = @Translation("Remote Media - Qualtrics"),
 *   field_types = {
 *     "string"
 *   }
 * )
 */
class MediaQualtricsEmbedFormatter extends MediaRemoteFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function getUrlRegexPattern() {
    return '/^https?:\/\/(?:[\w-]+\.)?qualtrics\.com(?:\/.*)?$/';
  }

  /**
   * {@inheritdoc}
   */
  public static function getValidUrlExampleStrings(): array {
    return [
      'https://[your-domain].qualtrics.com/[your-embed]',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function deriveMediaDefaultNameFromUrl($url) {
    $matches = [];
    $pattern = static::getUrlRegexPattern();
    preg_match_all($pattern, $url, $matches);
    if (!empty($matches[0][0])) {
      return t('Qualtrics from @url', [
        '@url' => $url,
      ]);
    }
    return parent::deriveMediaDefaultNameFromUrl($url);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      /** @var \Drupal\Core\Field\FieldItemInterface $item */
      if ($item->isEmpty()) {
        continue;
      }

      $matches = [];
      $pattern = static::getUrlRegexPattern();
      preg_match_all($pattern, $item->value, $matches);

      if (empty($matches[0][0])) {
        continue;
      }

      // Append a special query parameter so Qualtrics communicated the parent
      // page about height changes.
      // Note: this parameter is not public and can change at any point.
      $url = Url::fromUri($item->value, ['query' => ['Q_CHL' => 'si']]);
      $link = Link::fromTextAndUrl('public form', $url);
      $message = $this->t('If this embedded form is not working in your browser, please visit this @link to view it on qualtrics.com', ['@link' => $link->toString()]);

      $elements[$delta] = [
        '#theme' => 'media_qualtrics',
        '#url' => $url,
        '#title' => $item->getParent()->getParent()->get('name')->value,
        '#link' => $message,
        '#attached' => [
          'library' => [
            'media_qualtrics/qualtrics-controller',
          ],
        ],
      ];
    }
    return $elements;
  }

}
